import { TextDocument, Position } from 'vscode-languageserver-textdocument';
import type * as ts from 'typescript';
import { HTMLDocument } from 'vscode-html-languageservice';
import type * as emmet from 'vscode-emmet-helper';
export declare type DocumentLanguageService = ReturnType<typeof getDocumentLanguageService>;
export declare type LanguageService = ReturnType<typeof createLanguageService>;
export declare type LanguageServiceHost = ts.LanguageServiceHost & {
    getEmmetConfig?: (syntax: string) => Promise<emmet.VSCodeEmmetConfig> | emmet.VSCodeEmmetConfig;
};
export declare type Dependencies = {
    typescript: typeof import('typescript/lib/tsserverlibrary');
};
export declare function getDocumentLanguageService({ typescript: ts }: Dependencies): {
    doFormatting: (_document: TextDocument, options: import("vscode-languageserver").FormattingOptions) => import("vscode-languageserver").TextEdit[] | undefined;
    getFoldingRanges: (_document: TextDocument) => import("vscode-languageserver").FoldingRange[];
    doTagComplete: (document: TextDocument, position: import("vscode-languageserver").Position) => string | null | undefined;
    findLinkedEditingRanges: (document: TextDocument, position: import("vscode-languageserver").Position) => import("vscode-languageserver").LinkedEditingRanges | null;
};
export declare function createLanguageService({ typescript: ts }: Dependencies, vueHost: LanguageServiceHost, isTsPlugin?: boolean): {
    doValidation: (uri: string, response: (result: import("vscode-languageserver").Diagnostic[]) => void, isCancel?: (() => Promise<boolean>) | undefined) => Promise<void>;
    findDefinition: (uri: string, position: import("vscode-languageserver").Position) => import("vscode-languageserver").Location[] | import("vscode-languageserver").LocationLink[];
    findReferences: (uri: string, position: import("vscode-languageserver").Position) => import("vscode-languageserver").Location[];
    findTypeDefinition: (uri: string, position: import("vscode-languageserver").Position) => import("vscode-languageserver").LocationLink[];
    callHierarchy: {
        doPrepare: (uri: string, position: import("vscode-languageserver").Position) => import("vscode-languageserver").CallHierarchyItem[];
        getIncomingCalls: (item: import("vscode-languageserver").CallHierarchyItem) => import("vscode-languageserver").CallHierarchyIncomingCall[];
        getOutgoingCalls: (item: import("vscode-languageserver").CallHierarchyItem) => import("vscode-languageserver").CallHierarchyOutgoingCall[];
    };
    prepareRename: (uri: string, position: import("vscode-languageserver").Position) => import("vscode-languageserver").Range | import("vscode-languageserver").ResponseError<void> | undefined;
    doRename: (uri: string, position: import("vscode-languageserver").Position, newName: string) => import("vscode-languageserver").WorkspaceEdit | undefined;
    getEditsForFileRename: (oldUri: string, newUri: string) => import("vscode-languageserver").WorkspaceEdit | undefined;
    getSemanticTokens: (uri: string, range?: import("vscode-languageserver").Range | undefined, cancle?: import("vscode-languageserver").CancellationToken | undefined, resultProgress?: import("vscode-languageserver").ResultProgressReporter<import("vscode-languageserver").SemanticTokensPartialResult> | undefined) => import("vscode-languageserver").SemanticTokens | undefined;
    doHover: (uri: string, position: import("vscode-languageserver").Position) => import("vscode-languageserver").Hover | undefined;
    doComplete: (uri: string, position: import("vscode-languageserver-types").Position, context?: import("vscode-languageserver").CompletionContext | undefined, getNameCase?: {
        tag: () => Promise<"both" | "kebabCase" | "pascalCase">;
        attr: () => Promise<"kebabCase" | "pascalCase">;
    } | undefined) => Promise<import("vscode-languageserver-types").CompletionList | undefined>;
    getCodeActions: (uri: string, range: import("vscode-languageserver").Range, context: import("vscode-languageserver").CodeActionContext) => import("vscode-languageserver").CodeAction[];
    doCodeActionResolve: (codeAction: import("vscode-languageserver-types").CodeAction) => import("vscode-languageserver-types").CodeAction;
    doCompletionResolve: (item: import("vscode-languageserver").CompletionItem, newOffset?: number | undefined) => import("vscode-languageserver").CompletionItem;
    doCodeLensResolve: (codeLens: import("vscode-languageserver").CodeLens) => import("vscode-languageserver").CodeLens;
    getSignatureHelp: (uri: string, position: import("vscode-languageserver").Position) => import("vscode-languageserver").SignatureHelp | undefined;
    getSelectionRanges: (uri: string, positions: import("vscode-languageserver").Position[]) => import("vscode-languageserver").SelectionRange[] | undefined;
    getColorPresentations: (uri: string, color: import("vscode-languageserver").Color, range: import("vscode-languageserver").Range) => import("vscode-languageserver").ColorPresentation[] | undefined;
    getCodeLens: (uri: string) => import("vscode-languageserver").CodeLens[] | undefined;
    findDocumentHighlights: (uri: string, position: import("vscode-languageserver").Position) => import("vscode-languageserver").DocumentHighlight[] | undefined;
    findDocumentSymbols: (uri: string) => import("vscode-languageserver").SymbolInformation[] | undefined;
    findDocumentLinks: (uri: string) => Promise<import("vscode-languageserver").DocumentLink[] | undefined>;
    findDocumentColors: (uri: string) => import("vscode-languageserver").ColorInformation[] | undefined;
    dispose: () => void;
    __internal__: {
        rootPath: string;
        tsPlugin: Partial<ts.LanguageService>;
        tsProgramProxy: any;
        onInitProgress(cb: (p: number) => void): void;
        getTextDocument: (uri: string) => TextDocument | undefined;
        checkProject: () => boolean;
        getTsService: () => {
            findDefinition: (uri: string, position: import("vscode-languageserver").Position) => import("vscode-languageserver").LocationLink[];
            findTypeDefinition: (uri: string, position: import("vscode-languageserver").Position) => import("vscode-languageserver").LocationLink[];
            findReferences: (uri: string, position: import("vscode-languageserver").Position) => import("vscode-languageserver").Location[];
            prepareRename: (uri: string, position: import("vscode-languageserver").Position) => import("vscode-languageserver").Range | import("vscode-languageserver").ResponseError<void> | undefined;
            doRename: (uri: string, position: import("vscode-languageserver").Position, newName: string) => import("vscode-languageserver").WorkspaceEdit | undefined;
            getEditsForFileRename: (oldUri: string, newUri: string) => import("vscode-languageserver").WorkspaceEdit | undefined;
            getCodeActions: (uri: string, range: import("vscode-languageserver").Range, context: import("vscode-languageserver").CodeActionContext) => import("vscode-languageserver").CodeAction[] | undefined;
            doCodeActionResolve: (codeAction: import("vscode-languageserver").CodeAction) => import("vscode-languageserver").CodeAction;
            findDocumentHighlights: (uri: string, position: import("vscode-languageserver").Position) => import("vscode-languageserver").DocumentHighlight[];
            findDocumentSymbols: (uri: string) => import("vscode-languageserver").DocumentSymbol[];
            findWorkspaceSymbols: (uri: string) => import("vscode-languageserver").SymbolInformation[];
            doComplete: (uri: string, position: import("vscode-languageserver").Position, options?: ts.GetCompletionsAtPositionOptions | undefined) => import("vscode-languageserver").CompletionItem[];
            doCompletionResolve: (item: import("vscode-languageserver").CompletionItem, newOffset?: number | undefined) => import("vscode-languageserver").CompletionItem;
            doHover: (uri: string, position: import("vscode-languageserver").Position, documentOnly?: boolean | undefined) => import("vscode-languageserver").Hover | undefined;
            doFormatting: (uri: string, options: import("vscode-languageserver").FormattingOptions, range?: import("vscode-languageserver").Range | undefined) => import("vscode-languageserver").TextEdit[];
            getSignatureHelp: (uri: string, position: import("vscode-languageserver").Position) => import("vscode-languageserver").SignatureHelp | undefined;
            getSelectionRange: (uri: string, position: import("vscode-languageserver").Position) => import("vscode-languageserver").SelectionRange | undefined;
            doValidation: (uri: string, options: {
                semantic?: boolean | undefined;
                syntactic?: boolean | undefined;
                suggestion?: boolean | undefined;
                declaration?: boolean | undefined;
            }, cancellationToken?: ts.CancellationToken | undefined) => import("vscode-languageserver").Diagnostic[];
            getFoldingRanges: (uri: string) => import("vscode-languageserver").FoldingRange[];
            getDocumentSemanticTokens: (uri: string, range: import("vscode-languageserver").Range, cancle?: import("vscode-languageserver").CancellationToken | undefined) => [number, number, number, number, number][] | undefined;
            callHierarchy: {
                doPrepare: (uri: string, position: import("vscode-languageserver").Position) => import("vscode-languageserver").CallHierarchyItem[];
                getIncomingCalls: (item: import("vscode-languageserver").CallHierarchyItem) => import("vscode-languageserver").CallHierarchyIncomingCall[];
                getOutgoingCalls: (item: import("vscode-languageserver").CallHierarchyItem) => import("vscode-languageserver").CallHierarchyOutgoingCall[];
            };
            dispose: () => void;
            __internal__: {
                raw: import("typescript/lib/tsserverlibrary").LanguageService;
                host: {
                    getImportSuggestionsCache: () => any;
                    getCompilationSettings(): ts.CompilerOptions;
                    getNewLine?(): string;
                    getProjectVersion?(): string;
                    getScriptFileNames(): string[];
                    getScriptKind?(fileName: string): ts.ScriptKind;
                    getScriptVersion(fileName: string): string;
                    getScriptSnapshot(fileName: string): ts.IScriptSnapshot | undefined;
                    getProjectReferences?(): readonly ts.ProjectReference[] | undefined;
                    getLocalizedDiagnosticMessages?(): any;
                    getCancellationToken?(): ts.HostCancellationToken;
                    getCurrentDirectory(): string;
                    getDefaultLibFileName(options: ts.CompilerOptions): string;
                    log?(s: string): void;
                    trace?(s: string): void;
                    error?(s: string): void;
                    useCaseSensitiveFileNames?(): boolean;
                    readDirectory?(path: string, extensions?: readonly string[] | undefined, exclude?: readonly string[] | undefined, include?: readonly string[] | undefined, depth?: number | undefined): string[];
                    readFile?(path: string, encoding?: string | undefined): string | undefined;
                    realpath?(path: string): string;
                    fileExists?(path: string): boolean;
                    getTypeRootsVersion?(): number;
                    resolveModuleNames?(moduleNames: string[], containingFile: string, reusedNames: string[] | undefined, redirectedReference: ts.ResolvedProjectReference | undefined, options: ts.CompilerOptions): (ts.ResolvedModule | undefined)[];
                    getResolvedModuleWithFailedLookupLocationsFromCache?(modulename: string, containingFile: string): ts.ResolvedModuleWithFailedLookupLocations | undefined;
                    resolveTypeReferenceDirectives?(typeDirectiveNames: string[], containingFile: string, redirectedReference: ts.ResolvedProjectReference | undefined, options: ts.CompilerOptions): (ts.ResolvedTypeReferenceDirective | undefined)[];
                    getDirectories?(directoryName: string): string[];
                    getCustomTransformers?(): ts.CustomTransformers | undefined;
                    isKnownTypesPackageName?(name: string): boolean;
                    installPackage?(options: ts.InstallPackageOptions): Promise<ts.ApplyCodeActionCommandResult>;
                    writeFile?(fileName: string, content: string): void;
                    directoryExists?(directoryName: string): boolean;
                };
                getTextDocument: (uri: string) => TextDocument | undefined;
                getTextDocument2: (uri: string) => TextDocument | undefined;
            };
        };
        getGlobalDocs: () => TextDocument[];
        getSourceFile: (uri: string) => {
            uri: string;
            getTemplateTagNames: () => Set<string> | undefined;
            getTemplateAttrNames: () => Set<string> | undefined;
            getTextDocument: () => {
                readonly uri: string;
                readonly languageId: string;
                readonly version: number;
                getText: (range?: import("vscode-languageserver-textdocument").Range | undefined) => string;
                positionAt: (offset: number) => Position;
                offsetAt: (position: Position) => number;
                readonly lineCount: number;
            };
            update: (newVueDocument: TextDocument) => {
                scriptUpdated: boolean;
                templateScriptUpdated: boolean;
            };
            updateTemplateScript: () => boolean;
            getComponentCompletionData: () => Map<string, {
                item: import("vscode-languageserver").CompletionItem | undefined;
                bind: import("vscode-languageserver").CompletionItem[];
                on: import("vscode-languageserver").CompletionItem[];
                slot: import("vscode-languageserver").CompletionItem[];
            }>;
            getDiagnostics: (response: (diags: import("vscode-languageserver").Diagnostic[]) => void, isCancel?: (() => Promise<boolean>) | undefined) => Promise<void>;
            getTsSourceMaps: () => import(".").TsSourceMap[];
            getMainTsDoc: () => TextDocument;
            getCssSourceMaps: () => import(".").CssSourceMap[];
            getHtmlSourceMaps: () => import(".").HtmlSourceMap[];
            getPugSourceMaps: () => import(".").PugSourceMap[];
            getTemplateScriptData: () => {
                projectVersion: string | undefined;
                context: string[];
                componentItems: {
                    label: string;
                    labelDetails?: {
                        parameters?: string | undefined;
                        qualifier?: string | undefined;
                        type?: string | undefined;
                    } | undefined;
                    kind?: import("vscode-languageserver").CompletionItemKind | undefined;
                    tags?: 1[] | undefined;
                    detail?: string | undefined;
                    documentation?: string | {
                        kind: import("vscode-languageserver").MarkupKind;
                        value: string;
                    } | undefined;
                    deprecated?: boolean | undefined;
                    preselect?: boolean | undefined;
                    sortText?: string | undefined;
                    filterText?: string | undefined;
                    insertText?: string | undefined;
                    insertTextFormat?: import("vscode-languageserver").InsertTextFormat | undefined;
                    insertTextMode?: import("vscode-languageserver").InsertTextMode | undefined;
                    textEdit?: {
                        range: {
                            start: {
                                line: number;
                                character: number;
                            };
                            end: {
                                line: number;
                                character: number;
                            };
                        };
                        newText: string;
                    } | {
                        newText: string;
                        insert: {
                            start: {
                                line: number;
                                character: number;
                            };
                            end: {
                                line: number;
                                character: number;
                            };
                        };
                        replace: {
                            start: {
                                line: number;
                                character: number;
                            };
                            end: {
                                line: number;
                                character: number;
                            };
                        };
                    } | undefined;
                    additionalTextEdits?: {
                        range: {
                            start: {
                                line: number;
                                character: number;
                            };
                            end: {
                                line: number;
                                character: number;
                            };
                        };
                        newText: string;
                    }[] | undefined;
                    commitCharacters?: string[] | undefined;
                    command?: {
                        title: string;
                        command: string;
                        arguments?: any[] | undefined;
                    } | undefined;
                    data?: any;
                }[];
                components: string[];
                props: string[];
                setupReturns: string[];
                htmlElementItems: {
                    label: string;
                    labelDetails?: {
                        parameters?: string | undefined;
                        qualifier?: string | undefined;
                        type?: string | undefined;
                    } | undefined;
                    kind?: import("vscode-languageserver").CompletionItemKind | undefined;
                    tags?: 1[] | undefined;
                    detail?: string | undefined;
                    documentation?: string | {
                        kind: import("vscode-languageserver").MarkupKind;
                        value: string;
                    } | undefined;
                    deprecated?: boolean | undefined;
                    preselect?: boolean | undefined;
                    sortText?: string | undefined;
                    filterText?: string | undefined;
                    insertText?: string | undefined;
                    insertTextFormat?: import("vscode-languageserver").InsertTextFormat | undefined;
                    insertTextMode?: import("vscode-languageserver").InsertTextMode | undefined;
                    textEdit?: {
                        range: {
                            start: {
                                line: number;
                                character: number;
                            };
                            end: {
                                line: number;
                                character: number;
                            };
                        };
                        newText: string;
                    } | {
                        newText: string;
                        insert: {
                            start: {
                                line: number;
                                character: number;
                            };
                            end: {
                                line: number;
                                character: number;
                            };
                        };
                        replace: {
                            start: {
                                line: number;
                                character: number;
                            };
                            end: {
                                line: number;
                                character: number;
                            };
                        };
                    } | undefined;
                    additionalTextEdits?: {
                        range: {
                            start: {
                                line: number;
                                character: number;
                            };
                            end: {
                                line: number;
                                character: number;
                            };
                        };
                        newText: string;
                    }[] | undefined;
                    commitCharacters?: string[] | undefined;
                    command?: {
                        title: string;
                        command: string;
                        arguments?: any[] | undefined;
                    } | undefined;
                    data?: any;
                }[];
                htmlElements: string[];
            };
            getTeleports: () => import(".").TeleportSourceMap[];
            getDescriptor: () => {
                template: {
                    lang: string;
                    content: string;
                    loc: {
                        start: number;
                        end: number;
                    };
                } | null;
                script: {
                    lang: string;
                    content: string;
                    loc: {
                        start: number;
                        end: number;
                    };
                    src?: string | undefined;
                } | null;
                scriptSetup: {
                    lang: string;
                    content: string;
                    loc: {
                        start: number;
                        end: number;
                    };
                } | null;
                styles: {
                    lang: string;
                    content: string;
                    loc: {
                        start: number;
                        end: number;
                    };
                    module: boolean;
                    scoped: boolean;
                }[];
                customBlocks: {
                    lang: string;
                    content: string;
                    loc: {
                        start: number;
                        end: number;
                    };
                    type: string;
                }[];
            };
            getVueHtmlDocument: () => HTMLDocument;
            getTsDocuments: () => Map<string, TextDocument>;
            getVirtualScript: () => {
                document: TextDocument | undefined;
                sourceMap: import(".").TsSourceMap | undefined;
            };
            getScriptSetupData: () => {
                labels: {
                    start: number;
                    end: number;
                    binarys: {
                        parent: {
                            start: number;
                            end: number;
                        };
                        vars: {
                            isShortand: boolean;
                            inRoot: boolean;
                            start: number;
                            end: number;
                        }[];
                        left: {
                            start: number;
                            end: number;
                        };
                        right: {
                            start: number;
                            end: number;
                            isComputedCall: boolean;
                            withoutAs: {
                                start: number;
                                end: number;
                            };
                            as: {
                                start: number;
                                end: number;
                            } | undefined;
                        } | undefined;
                    }[];
                    label: {
                        start: number;
                        end: number;
                    };
                    parent: {
                        start: number;
                        end: number;
                    };
                }[];
                returnVarNames: {
                    start: number;
                    end: number;
                }[];
                defineProps: {
                    start: number;
                    end: number;
                    args?: {
                        start: number;
                        end: number;
                    } | undefined;
                    typeArgs?: {
                        start: number;
                        end: number;
                    } | undefined;
                } | undefined;
                defineEmit: {
                    start: number;
                    end: number;
                    args?: {
                        start: number;
                        end: number;
                    } | undefined;
                    typeArgs?: {
                        start: number;
                        end: number;
                    } | undefined;
                } | undefined;
                dollars: number[];
            } | undefined;
            getScriptsRaw: () => {
                documents: TextDocument[];
                sourceMaps: import(".").TsSourceMap[];
            };
            getTemplateScriptFormat: () => {
                document: TextDocument | undefined;
                sourceMap: import(".").TsSourceMap | undefined;
            };
            shouldVerifyTsScript: (tsUri: string, mode: 2 | 1 | 3 | 4) => "all" | "none" | "unused";
        } | undefined;
        getAllSourceFiles: () => {
            uri: string;
            getTemplateTagNames: () => Set<string> | undefined;
            getTemplateAttrNames: () => Set<string> | undefined;
            getTextDocument: () => {
                readonly uri: string;
                readonly languageId: string;
                readonly version: number;
                getText: (range?: import("vscode-languageserver-textdocument").Range | undefined) => string;
                positionAt: (offset: number) => Position;
                offsetAt: (position: Position) => number;
                readonly lineCount: number;
            };
            update: (newVueDocument: TextDocument) => {
                scriptUpdated: boolean;
                templateScriptUpdated: boolean;
            };
            updateTemplateScript: () => boolean;
            getComponentCompletionData: () => Map<string, {
                item: import("vscode-languageserver").CompletionItem | undefined;
                bind: import("vscode-languageserver").CompletionItem[];
                on: import("vscode-languageserver").CompletionItem[];
                slot: import("vscode-languageserver").CompletionItem[];
            }>;
            getDiagnostics: (response: (diags: import("vscode-languageserver").Diagnostic[]) => void, isCancel?: (() => Promise<boolean>) | undefined) => Promise<void>;
            getTsSourceMaps: () => import(".").TsSourceMap[];
            getMainTsDoc: () => TextDocument;
            getCssSourceMaps: () => import(".").CssSourceMap[];
            getHtmlSourceMaps: () => import(".").HtmlSourceMap[];
            getPugSourceMaps: () => import(".").PugSourceMap[];
            getTemplateScriptData: () => {
                projectVersion: string | undefined;
                context: string[];
                componentItems: {
                    label: string;
                    labelDetails?: {
                        parameters?: string | undefined;
                        qualifier?: string | undefined;
                        type?: string | undefined;
                    } | undefined;
                    kind?: import("vscode-languageserver").CompletionItemKind | undefined;
                    tags?: 1[] | undefined;
                    detail?: string | undefined;
                    documentation?: string | {
                        kind: import("vscode-languageserver").MarkupKind;
                        value: string;
                    } | undefined;
                    deprecated?: boolean | undefined;
                    preselect?: boolean | undefined;
                    sortText?: string | undefined;
                    filterText?: string | undefined;
                    insertText?: string | undefined;
                    insertTextFormat?: import("vscode-languageserver").InsertTextFormat | undefined;
                    insertTextMode?: import("vscode-languageserver").InsertTextMode | undefined;
                    textEdit?: {
                        range: {
                            start: {
                                line: number;
                                character: number;
                            };
                            end: {
                                line: number;
                                character: number;
                            };
                        };
                        newText: string;
                    } | {
                        newText: string;
                        insert: {
                            start: {
                                line: number;
                                character: number;
                            };
                            end: {
                                line: number;
                                character: number;
                            };
                        };
                        replace: {
                            start: {
                                line: number;
                                character: number;
                            };
                            end: {
                                line: number;
                                character: number;
                            };
                        };
                    } | undefined;
                    additionalTextEdits?: {
                        range: {
                            start: {
                                line: number;
                                character: number;
                            };
                            end: {
                                line: number;
                                character: number;
                            };
                        };
                        newText: string;
                    }[] | undefined;
                    commitCharacters?: string[] | undefined;
                    command?: {
                        title: string;
                        command: string;
                        arguments?: any[] | undefined;
                    } | undefined;
                    data?: any;
                }[];
                components: string[];
                props: string[];
                setupReturns: string[];
                htmlElementItems: {
                    label: string;
                    labelDetails?: {
                        parameters?: string | undefined;
                        qualifier?: string | undefined;
                        type?: string | undefined;
                    } | undefined;
                    kind?: import("vscode-languageserver").CompletionItemKind | undefined;
                    tags?: 1[] | undefined;
                    detail?: string | undefined;
                    documentation?: string | {
                        kind: import("vscode-languageserver").MarkupKind;
                        value: string;
                    } | undefined;
                    deprecated?: boolean | undefined;
                    preselect?: boolean | undefined;
                    sortText?: string | undefined;
                    filterText?: string | undefined;
                    insertText?: string | undefined;
                    insertTextFormat?: import("vscode-languageserver").InsertTextFormat | undefined;
                    insertTextMode?: import("vscode-languageserver").InsertTextMode | undefined;
                    textEdit?: {
                        range: {
                            start: {
                                line: number;
                                character: number;
                            };
                            end: {
                                line: number;
                                character: number;
                            };
                        };
                        newText: string;
                    } | {
                        newText: string;
                        insert: {
                            start: {
                                line: number;
                                character: number;
                            };
                            end: {
                                line: number;
                                character: number;
                            };
                        };
                        replace: {
                            start: {
                                line: number;
                                character: number;
                            };
                            end: {
                                line: number;
                                character: number;
                            };
                        };
                    } | undefined;
                    additionalTextEdits?: {
                        range: {
                            start: {
                                line: number;
                                character: number;
                            };
                            end: {
                                line: number;
                                character: number;
                            };
                        };
                        newText: string;
                    }[] | undefined;
                    commitCharacters?: string[] | undefined;
                    command?: {
                        title: string;
                        command: string;
                        arguments?: any[] | undefined;
                    } | undefined;
                    data?: any;
                }[];
                htmlElements: string[];
            };
            getTeleports: () => import(".").TeleportSourceMap[];
            getDescriptor: () => {
                template: {
                    lang: string;
                    content: string;
                    loc: {
                        start: number;
                        end: number;
                    };
                } | null;
                script: {
                    lang: string;
                    content: string;
                    loc: {
                        start: number;
                        end: number;
                    };
                    src?: string | undefined;
                } | null;
                scriptSetup: {
                    lang: string;
                    content: string;
                    loc: {
                        start: number;
                        end: number;
                    };
                } | null;
                styles: {
                    lang: string;
                    content: string;
                    loc: {
                        start: number;
                        end: number;
                    };
                    module: boolean;
                    scoped: boolean;
                }[];
                customBlocks: {
                    lang: string;
                    content: string;
                    loc: {
                        start: number;
                        end: number;
                    };
                    type: string;
                }[];
            };
            getVueHtmlDocument: () => HTMLDocument;
            getTsDocuments: () => Map<string, TextDocument>;
            getVirtualScript: () => {
                document: TextDocument | undefined;
                sourceMap: import(".").TsSourceMap | undefined;
            };
            getScriptSetupData: () => {
                labels: {
                    start: number;
                    end: number;
                    binarys: {
                        parent: {
                            start: number;
                            end: number;
                        };
                        vars: {
                            isShortand: boolean;
                            inRoot: boolean;
                            start: number;
                            end: number;
                        }[];
                        left: {
                            start: number;
                            end: number;
                        };
                        right: {
                            start: number;
                            end: number;
                            isComputedCall: boolean;
                            withoutAs: {
                                start: number;
                                end: number;
                            };
                            as: {
                                start: number;
                                end: number;
                            } | undefined;
                        } | undefined;
                    }[];
                    label: {
                        start: number;
                        end: number;
                    };
                    parent: {
                        start: number;
                        end: number;
                    };
                }[];
                returnVarNames: {
                    start: number;
                    end: number;
                }[];
                defineProps: {
                    start: number;
                    end: number;
                    args?: {
                        start: number;
                        end: number;
                    } | undefined;
                    typeArgs?: {
                        start: number;
                        end: number;
                    } | undefined;
                } | undefined;
                defineEmit: {
                    start: number;
                    end: number;
                    args?: {
                        start: number;
                        end: number;
                    } | undefined;
                    typeArgs?: {
                        start: number;
                        end: number;
                    } | undefined;
                } | undefined;
                dollars: number[];
            } | undefined;
            getScriptsRaw: () => {
                documents: TextDocument[];
                sourceMaps: import(".").TsSourceMap[];
            };
            getTemplateScriptFormat: () => {
                document: TextDocument | undefined;
                sourceMap: import(".").TsSourceMap | undefined;
            };
            shouldVerifyTsScript: (tsUri: string, mode: 2 | 1 | 3 | 4) => "all" | "none" | "unused";
        }[];
        getD3: (document: TextDocument) => Promise<string>;
        executeCommand: (uri: string, command: string, args: any[] | undefined, connection: import("vscode-languageserver").Connection) => Promise<void>;
        detectTagNameCase: (uri: string) => {
            tag: "both" | "kebabCase" | "pascalCase" | "unsure";
            attr: "kebabCase" | "pascalCase" | "unsure";
        };
        doRefAutoClose: (document: TextDocument, position: import("vscode-languageserver").Position) => string | null | undefined;
    };
};
