"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.register = void 0;
const node_1 = require("vscode-languageserver/node");
const languageServices = require("../utils/languageServices");
function register({ sourceFiles }) {
    return (uri, color, range) => {
        const sourceFile = sourceFiles.get(uri);
        if (!sourceFile)
            return;
        const cssResult = getCssResult(sourceFile);
        return cssResult;
        function getCssResult(sourceFile) {
            let result = [];
            for (const sourceMap of sourceFile.getCssSourceMaps()) {
                const cssLanguageService = languageServices.getCssLanguageService(sourceMap.mappedDocument.languageId);
                if (!cssLanguageService || !sourceMap.stylesheet)
                    continue;
                const cssRanges = sourceMap.getMappedRanges(range.start, range.end);
                for (const cssRange of cssRanges) {
                    const _result = cssLanguageService.getColorPresentations(sourceMap.mappedDocument, sourceMap.stylesheet, color, cssRange);
                    for (const item of _result) {
                        if (item.textEdit) {
                            if (node_1.TextEdit.is(item.textEdit)) {
                                const vueRange = sourceMap.getSourceRange(item.textEdit.range.start, item.textEdit.range.end);
                                if (vueRange) {
                                    item.textEdit.range = vueRange;
                                }
                            }
                            if (item.additionalTextEdits) {
                                for (const textEdit of item.additionalTextEdits) {
                                    const vueRange = sourceMap.getSourceRange(item.textEdit.range.start, item.textEdit.range.end);
                                    if (vueRange) {
                                        textEdit.range = vueRange;
                                    }
                                }
                            }
                        }
                    }
                    result = result.concat(_result);
                }
            }
            return result;
        }
    };
}
exports.register = register;
//# sourceMappingURL=colorPresentation.js.map