import type * as ts2 from 'vscode-typescript-languageservice';
import * as css from 'vscode-css-languageservice';
import type { DocumentContext } from 'vscode-html-languageservice';
import { TextDocument } from 'vscode-languageserver-textdocument';
import { CompletionItem, Diagnostic } from 'vscode-languageserver/node';
import { TsSourceMap } from './utils/sourceMaps';
export declare const defaultLanguages: {
    template: string;
    script: string;
    style: string;
};
export declare type SourceFile = ReturnType<typeof createSourceFile>;
export declare function createSourceFile(initialDocument: TextDocument, tsLanguageService: ts2.LanguageService, ts: typeof import('typescript'), styleMode: 'api' | 'format', documentContext: DocumentContext | undefined): {
    uri: string;
    getTemplateTagNames: () => Set<string> | undefined;
    getTemplateAttrNames: () => Set<string> | undefined;
    getTextDocument: () => {
        readonly uri: string;
        readonly languageId: string;
        readonly version: number;
        getText: (range?: import("vscode-languageserver-textdocument").Range | undefined) => string;
        positionAt: (offset: number) => import("vscode-languageserver-textdocument").Position;
        offsetAt: (position: import("vscode-languageserver-textdocument").Position) => number;
        readonly lineCount: number;
    };
    update: (newVueDocument: TextDocument) => {
        scriptUpdated: boolean;
        templateScriptUpdated: boolean;
    };
    updateTemplateScript: () => boolean;
    getComponentCompletionData: () => Map<string, {
        item: CompletionItem | undefined;
        bind: CompletionItem[];
        on: CompletionItem[];
        slot: CompletionItem[];
    }>;
    getDiagnostics: (response: (diags: Diagnostic[]) => void, isCancel?: (() => Promise<boolean>) | undefined) => Promise<void>;
    getTsSourceMaps: () => TsSourceMap[];
    getMainTsDoc: () => css.TextDocument;
    getCssSourceMaps: () => import("./utils/sourceMaps").CssSourceMap[];
    getHtmlSourceMaps: () => import("./utils/sourceMaps").HtmlSourceMap[];
    getPugSourceMaps: () => import("./utils/sourceMaps").PugSourceMap[];
    getTemplateScriptData: () => {
        projectVersion: string | undefined;
        context: string[];
        componentItems: {
            label: string;
            labelDetails?: {
                parameters?: string | undefined;
                qualifier?: string | undefined;
                type?: string | undefined;
            } | undefined;
            kind?: import("vscode-languageserver/node").CompletionItemKind | undefined;
            tags?: 1[] | undefined;
            detail?: string | undefined;
            documentation?: string | {
                kind: import("vscode-languageserver/node").MarkupKind;
                value: string;
            } | undefined;
            deprecated?: boolean | undefined;
            preselect?: boolean | undefined;
            sortText?: string | undefined;
            filterText?: string | undefined;
            insertText?: string | undefined;
            insertTextFormat?: import("vscode-languageserver/node").InsertTextFormat | undefined;
            insertTextMode?: import("vscode-languageserver/node").InsertTextMode | undefined;
            textEdit?: {
                range: {
                    start: {
                        line: number;
                        character: number;
                    };
                    end: {
                        line: number;
                        character: number;
                    };
                };
                newText: string;
            } | {
                newText: string;
                insert: {
                    start: {
                        line: number;
                        character: number;
                    };
                    end: {
                        line: number;
                        character: number;
                    };
                };
                replace: {
                    start: {
                        line: number;
                        character: number;
                    };
                    end: {
                        line: number;
                        character: number;
                    };
                };
            } | undefined;
            additionalTextEdits?: {
                range: {
                    start: {
                        line: number;
                        character: number;
                    };
                    end: {
                        line: number;
                        character: number;
                    };
                };
                newText: string;
            }[] | undefined;
            commitCharacters?: string[] | undefined;
            command?: {
                title: string;
                command: string;
                arguments?: any[] | undefined;
            } | undefined;
            data?: any;
        }[];
        components: string[];
        props: string[];
        setupReturns: string[];
        htmlElementItems: {
            label: string;
            labelDetails?: {
                parameters?: string | undefined;
                qualifier?: string | undefined;
                type?: string | undefined;
            } | undefined;
            kind?: import("vscode-languageserver/node").CompletionItemKind | undefined;
            tags?: 1[] | undefined;
            detail?: string | undefined;
            documentation?: string | {
                kind: import("vscode-languageserver/node").MarkupKind;
                value: string;
            } | undefined;
            deprecated?: boolean | undefined;
            preselect?: boolean | undefined;
            sortText?: string | undefined;
            filterText?: string | undefined;
            insertText?: string | undefined;
            insertTextFormat?: import("vscode-languageserver/node").InsertTextFormat | undefined;
            insertTextMode?: import("vscode-languageserver/node").InsertTextMode | undefined;
            textEdit?: {
                range: {
                    start: {
                        line: number;
                        character: number;
                    };
                    end: {
                        line: number;
                        character: number;
                    };
                };
                newText: string;
            } | {
                newText: string;
                insert: {
                    start: {
                        line: number;
                        character: number;
                    };
                    end: {
                        line: number;
                        character: number;
                    };
                };
                replace: {
                    start: {
                        line: number;
                        character: number;
                    };
                    end: {
                        line: number;
                        character: number;
                    };
                };
            } | undefined;
            additionalTextEdits?: {
                range: {
                    start: {
                        line: number;
                        character: number;
                    };
                    end: {
                        line: number;
                        character: number;
                    };
                };
                newText: string;
            }[] | undefined;
            commitCharacters?: string[] | undefined;
            command?: {
                title: string;
                command: string;
                arguments?: any[] | undefined;
            } | undefined;
            data?: any;
        }[];
        htmlElements: string[];
    };
    getTeleports: () => import("./utils/sourceMaps").TeleportSourceMap[];
    getDescriptor: () => {
        template: {
            lang: string;
            content: string;
            loc: {
                start: number;
                end: number;
            };
        } | null;
        script: {
            lang: string;
            content: string;
            loc: {
                start: number;
                end: number;
            };
            src?: string | undefined;
        } | null;
        scriptSetup: {
            lang: string;
            content: string;
            loc: {
                start: number;
                end: number;
            };
        } | null;
        styles: {
            lang: string;
            content: string;
            loc: {
                start: number;
                end: number;
            };
            module: boolean;
            scoped: boolean;
        }[];
        customBlocks: {
            lang: string;
            content: string;
            loc: {
                start: number;
                end: number;
            };
            type: string;
        }[];
    };
    getVueHtmlDocument: () => import("vscode-html-languageservice").HTMLDocument;
    getTsDocuments: () => Map<string, css.TextDocument>;
    getVirtualScript: () => {
        document: css.TextDocument | undefined;
        sourceMap: TsSourceMap | undefined;
    };
    getScriptSetupData: () => {
        labels: {
            start: number;
            end: number;
            binarys: {
                parent: {
                    start: number;
                    end: number;
                };
                vars: {
                    isShortand: boolean;
                    inRoot: boolean;
                    start: number;
                    end: number;
                }[];
                left: {
                    start: number;
                    end: number;
                };
                right: {
                    start: number;
                    end: number;
                    isComputedCall: boolean;
                    withoutAs: {
                        start: number;
                        end: number;
                    };
                    as: {
                        start: number;
                        end: number;
                    } | undefined;
                } | undefined;
            }[];
            label: {
                start: number;
                end: number;
            };
            parent: {
                start: number;
                end: number;
            };
        }[];
        returnVarNames: {
            start: number;
            end: number;
        }[];
        defineProps: {
            start: number;
            end: number;
            args?: {
                start: number;
                end: number;
            } | undefined;
            typeArgs?: {
                start: number;
                end: number;
            } | undefined;
        } | undefined;
        defineEmit: {
            start: number;
            end: number;
            args?: {
                start: number;
                end: number;
            } | undefined;
            typeArgs?: {
                start: number;
                end: number;
            } | undefined;
        } | undefined;
        dollars: number[];
    } | undefined;
    getScriptsRaw: () => {
        documents: css.TextDocument[];
        sourceMaps: TsSourceMap[];
    };
    getTemplateScriptFormat: () => {
        document: css.TextDocument | undefined;
        sourceMap: TsSourceMap | undefined;
    };
    shouldVerifyTsScript: (tsUri: string, mode: 1 | 2 | 3 | 4) => 'all' | 'none' | 'unused';
};
